import threading

_tls = threading.local()

_tls.INSTALLED_POOLS = {}


def configure(pools):
    _tls.INSTALLED_POOLS = dict(pools)


def get_installed_pools():
    return _tls.INSTALLED_POOLS


try:
    from django.conf import settings
    configure(pools=settings.POOL_OF_RAMOS)
except ImportError:
    try:
        from simple_settings import settings
        configure(pools=settings.POOL_OF_RAMOS)
    except ImportError:
        pass


try:
    from django.core.exceptions import ImproperlyConfigured
except ImportError:
    class ImproperlyConfigured(Exception):
        pass


try:
    from django.utils.module_loading import import_string
except ImportError:
    from importlib import import_module

    def import_string(dotted_path):
        """
        Import a dotted module path and return the attribute/class designated
        by the last name in the path. Raise ImportError if the import failed.
        """

        try:
            module_path, class_name = dotted_path.rsplit('.', 1)
        except ValueError:
            raise ImportError(
                "{} doesn't look like a module path".format(
                    dotted_path
                )
            )

        module = import_module(module_path)

        try:
            return getattr(module, class_name)
        except AttributeError:
            raise ImportError(
                'Module "{}" does not define a "{}" attribute/class'.format(
                    module_path,
                    class_name
                )
            )



# -*- coding: utf-8 -*-


class ThreadSafeCreateMixin(object):
    """
    ThreadSafeCreateMixin can be used as an inheritance
    of thread safe backend implementations
    """

    @classmethod
    def create(cls):
        """
        Return always a new instance of the backend class
        """
        return cls()


class SingletonCreateMixin(object):
    """
    SingletonCreateMixin can be used as an inheritance
    of singleton backend implementations
    """

    _instances = {}

    @classmethod
    def create(cls):
        """
        Return always the same instance of the backend class
        """
        if cls not in cls._instances:
            cls._instances[cls] = cls()

        return cls._instances[cls]



# -*- coding: utf-8 -*-

from .compat import ImproperlyConfigured, get_installed_pools, import_string
from .exceptions import InvalidBackendError


class BackendPool(object):
    """
    BackendPool is an interface to get instances of backend types
    """

    backend_type = None

    @classmethod
    def get(cls, backend_id):
        """
        Return an instance of backend type
        """

        for backend_class in cls._get_backends_classes():
            if backend_class.id == backend_id:
                return backend_class.create()

        raise InvalidBackendError(
            cls.backend_type,
            backend_id,
            get_installed_pools()[cls.backend_type]
        )

    @classmethod
    def all(cls):
        """
        Return a list of instances of backend type
        """

        return [
            backend_class.create()
            for backend_class in cls._get_backends_classes()
        ]

    @classmethod
    def _get_backends_classes(cls):
        try:
            backend_list = get_installed_pools()[cls.backend_type]
        except KeyError:
            raise ImproperlyConfigured(
                u'Backend type "{}" config not found'.format(cls.backend_type)
            )

        return [
            cls._get_backend_class(backend_path)
            for backend_path in backend_list
        ]

    @classmethod
    def _get_backend_class(cls, backend_path):
        backend_class = import_string(backend_path)

        return backend_class



import os

from setuptools import setup


def read(fname):
    path = os.path.join(os.path.dirname(__file__), fname)
    with open(path) as f:
        return f.read()


setup(
    name='ramos',
    version='1.1.0',
    description=(
        'Generic backend pool '
    ),
    long_description=read('README.rst'),
    author='LuizaLabs',
    author_email='pypi@luizalabs.com',
    url='https://github.com/luizalabs/ramos',
    keywords='django backends pool',
    packages=['ramos'],
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
    ],
    extras_require={
        'django': ['Django>=1.8'],
        'simple_settings': ['simple_settings']
    }
)



# -*- coding: utf-8 -*-
class InvalidBackendError(Exception):
    """
    InvalidBackendError is raised when instances of BackendPool
    can't get a backend implementation
    """

    def __init__(self, backend_type, backend_id, available_backends):
        self.backend_type = backend_type
        self.backend_id = backend_id
        self.available_backends = available_backends
        self.error_message = u'Invalid {} backend: {}'.format(
            self.backend_type, self.backend_id
        )

        super(InvalidBackendError, self).__init__(
            self.backend_type,
            self.backend_id,
            self.error_message,
            self.available_backends,
        )
